//
//
//	DXF file lexer class
//
//											(C) 1997 JoOl



#include "LexerDxf.h"



LexerDxf::LexerDxf(int32 BufLen, int32 LexmLen): LexerText(BufLen, LexmLen)
{
	SetCharset();
	SetLexemes();
}


LexerDxf::~LexerDxf()
{
}


void
LexerDxf::SetCharset()
{
	SetDefaultCharset();

	SetChar('$', LX_CHAR_ALPHA);
	SetChar(0x0d, LX_CHAR_SPACE);
}



void
LexerDxf::SetLexemes()
{
	AddLexeme(new Lexeme("SECTION", LX_TOKEN_SECTION));
		AddLexeme(new Lexeme("HEADER", LX_TOKEN_HEADER));
			AddLexeme(new Lexeme("$ACADVER", LX_TOKEN_S_ACADVER));
		AddLexeme(new Lexeme("TABLES", LX_TOKEN_TABLES));
			AddLexeme(new Lexeme("TABLE", LX_TOKEN_TABLE));
				AddLexeme(new Lexeme("APPID", LX_TOKEN_APPID));
//				AddLexeme(new Lexeme("DIMSTYLE", LX_TOKEN_DIMSTYLE));
//				AddLexeme(new Lexeme("LTYPE", LX_TOKEN_LTYPE));
				AddLexeme(new Lexeme("LAYER", LX_TOKEN_LAYER));
//				AddLexeme(new Lexeme("STYLE", LX_TOKEN_STYLE));
//				AddLexeme(new Lexeme("UCS", LX_TOKEN_UCS));
				AddLexeme(new Lexeme("VIEW", LX_TOKEN_VIEW));
//				AddLexeme(new Lexeme("VPORT", LX_TOKEN_VPORT));
			AddLexeme(new Lexeme("ENDTAB", LX_TOKEN_ENDTAB));
		AddLexeme(new Lexeme("BLOCKS", LX_TOKEN_BLOCKS));
		AddLexeme(new Lexeme("ENTITIES", LX_TOKEN_ENTITIES));
			AddLexeme(new Lexeme("LINE", LX_TOKEN_LINE));
			AddLexeme(new Lexeme("POINT", LX_TOKEN_POINT));
			AddLexeme(new Lexeme("VERTEX", LX_TOKEN_VERTEX));
			AddLexeme(new Lexeme("3DFACE", LX_TOKEN_3DFACE));
			AddLexeme(new Lexeme("POLYLINE", LX_TOKEN_POLYLINE));
			AddLexeme(new Lexeme("SEQEND", LX_TOKEN_SEQEND));
	AddLexeme(new Lexeme("ENDSEC", LX_TOKEN_ENDSEC));
	AddLexeme(new Lexeme("EOF", LX_TOKEN_EOF));

	SortLexemes();
}


bool
LexerDxf::ReadLine()
{
	while ((!EndOfFile)
	&& (Charset[uint8(*LexmEnd)] == LX_CHAR_SPACE)
	&& (*LexmEnd != 0x0d))
	{
		FWD;
	}
	LexmStart = LexmEnd;
	while ((!EndOfFile) && (*LexmEnd != 0x0d))
	{
		FWD;
	}

	char*			L = GetLexeme();
	uint8			CharType = Charset[uint8(*L)];
	if ((CharType == LX_CHAR_DIGIT) || (*L == '-'))
		Token = LX_TYPE_INTEGER;
	else if (*L == '.')
		Token = LX_TYPE_FLOAT;
	else
		Token = LX_TYPE_STRING;

	// Check if the lexeme is an integer, a float or a string
	char*			Ptr = L;
	while ((*Ptr) && (Token != LX_TYPE_STRING))
	{
		CharType = Charset[uint8(*Ptr)];
		if (Token == LX_TYPE_INTEGER)
		{						// Try to upgrade an integer as a float
			if (*Ptr == '-')
			{}
			else if ((*Ptr == '.') || (*Ptr == 'e') || (*Ptr == 'E'))
				Token = LX_TYPE_FLOAT;
			else if (CharType != LX_CHAR_DIGIT)
				Token = LX_TYPE_STRING;
		}
		else		// Try to upgrade a float as a string
		{
			if ((*Ptr != '.') && (*Ptr != '-')
			&& (*Ptr != '+') && (*Ptr != 'e')
			&& (*Ptr != 'E') && (CharType != LX_CHAR_DIGIT))
				Token = LX_TYPE_STRING;
		}
		Ptr++;
	}

	if (Token == LX_TYPE_INTEGER)
	{
		TokenValue.Int = atoi(L);
		DeleteLexeme(L);
	}
	else if (Token == LX_TYPE_FLOAT)
	{
		TokenValue.Float = atof(L);
		DeleteLexeme(L);
	}
	else	 // string / lexeme
	{
		TokenValue.Str = L;
		Lexeme*		Lexem = SearchLexeme(L);
		if (Lexem)
			Token = Lexem->GetToken();
	}

	return true;
}



bool
LexerDxf::NextToken(uint32& Out)
{
	InitNextToken();

	if (EndOfFile)
	{
		Out = Token = LX_TOKEN_EOF;
		return true;
	}

	// Group code
	if (!SkipSpaces())
		return false;
	LexmStart = LexmEnd;
	if (!LexNumber())
		return false;
	if (Token != LX_TYPE_INTEGER)
		return false;
	GroupCode = TokenValue.Int;

	// Group data
	if (!SkipLine())
		return false;
	LexmStart = LexmEnd;
	if (!ReadLine())
		return false;
	Out = Token;
	return true;
}
