//
//
//	OBJ file lexer class
//
//											(C) 1997 JoOl



#include "LexerObj.h"



LexerObj::LexerObj(int32 BufLen, int32 LexmLen): LexerText(BufLen, LexmLen)
{
	SetCharset();
	SetLexemes();
}


LexerObj::~LexerObj()
{
}


void
LexerObj::SetCharset()
{
	SetDefaultCharset();

	SetChar('_', LX_CHAR_ALPHA);
	SetChar('/', LX_CHAR_SLASH);
	SetChar('-', LX_CHAR_MINUS);

	SetChar('#', LX_CHAR_COMMENT);
	SetChar('$', LX_CHAR_COMMENT);		// ? 32_dodge.obj
}



void
LexerObj::SetLexemes()
{
	AddLexeme(new Lexeme("v", LX_TOKEN_V));
	AddLexeme(new Lexeme("vt", LX_TOKEN_VTEXTURE));
	AddLexeme(new Lexeme("vn", LX_TOKEN_VNORMAL));
	AddLexeme(new Lexeme("vp", LX_TOKEN_VPARAM));
	AddLexeme(new Lexeme("cstype", LX_TOKEN_CSTYPE));
		AddLexeme(new Lexeme("rat", LX_TOKEN_RAT));
		AddLexeme(new Lexeme("bmatrix", LX_TOKEN_BMATRIX));
		AddLexeme(new Lexeme("bezier", LX_TOKEN_BEZIER));
		AddLexeme(new Lexeme("bspline", LX_TOKEN_BSPLINE));
		AddLexeme(new Lexeme("cardinal", LX_TOKEN_CARDINAL));
		AddLexeme(new Lexeme("taylor", LX_TOKEN_TAYLOR));
	AddLexeme(new Lexeme("deg", LX_TOKEN_DEGREE));
	AddLexeme(new Lexeme("bmat_u", LX_TOKEN_BMATU));
	AddLexeme(new Lexeme("bmat_v", LX_TOKEN_BMATV));
	AddLexeme(new Lexeme("step", LX_TOKEN_STEP));

	AddLexeme(new Lexeme("p", LX_TOKEN_POINT));
	AddLexeme(new Lexeme("l", LX_TOKEN_LINE));
	AddLexeme(new Lexeme("f", LX_TOKEN_FACE));
	AddLexeme(new Lexeme("fo", LX_TOKEN_FACE));		// for compatibility
	AddLexeme(new Lexeme("curv", LX_TOKEN_CURVE));
	AddLexeme(new Lexeme("curv2", LX_TOKEN_CURVE2D));
	AddLexeme(new Lexeme("surf", LX_TOKEN_SURFACE));

	AddLexeme(new Lexeme("parm_u", LX_TOKEN_PARAMU));
	AddLexeme(new Lexeme("parm_v", LX_TOKEN_PARAMV));
	AddLexeme(new Lexeme("trim", LX_TOKEN_TRIM));
	AddLexeme(new Lexeme("hole", LX_TOKEN_HOLE));
	AddLexeme(new Lexeme("scrv", LX_TOKEN_SCURVE));
	AddLexeme(new Lexeme("sp", LX_TOKEN_SPOINT));
	AddLexeme(new Lexeme("end", LX_TOKEN_ENDSTMT));

	AddLexeme(new Lexeme("con", LX_TOKEN_CONNECT));

	AddLexeme(new Lexeme("g", LX_TOKEN_GROUPNAME));
	AddLexeme(new Lexeme("s", LX_TOKEN_SMOOTHGROUP));
	AddLexeme(new Lexeme("mg", LX_TOKEN_MERGEGROUP));
	AddLexeme(new Lexeme("o", LX_TOKEN_OBJECTNAME));

	AddLexeme(new Lexeme("bevel", LX_TOKEN_BEVELINTERP));
	AddLexeme(new Lexeme("c_interp", LX_TOKEN_CINTERP));
	AddLexeme(new Lexeme("d_interp", LX_TOKEN_DINTERP));

	AddLexeme(new Lexeme("lod", LX_TOKEN_LOD));
	AddLexeme(new Lexeme("maplib", LX_TOKEN_MAPLIB));
	AddLexeme(new Lexeme("usemap", LX_TOKEN_USEMAP));
	AddLexeme(new Lexeme("usemtl", LX_TOKEN_USEMATERIAL));
	AddLexeme(new Lexeme("mtllib", LX_TOKEN_MATLIB));
	AddLexeme(new Lexeme("shadow_obj", LX_TOKEN_SHADOW));
	AddLexeme(new Lexeme("trace_obj", LX_TOKEN_TRACE));
	AddLexeme(new Lexeme("ctech", LX_TOKEN_CTECH));
	AddLexeme(new Lexeme("stech", LX_TOKEN_STECH));
		AddLexeme(new Lexeme("cparm", LX_TOKEN_CPARM));
		AddLexeme(new Lexeme("cspace", LX_TOKEN_CSPACE));
		AddLexeme(new Lexeme("cparma", LX_TOKEN_CPARMA));
		AddLexeme(new Lexeme("cparmb", LX_TOKEN_CPARMB));

	AddLexeme(new Lexeme("bsp", LX_TOKEN_BSP));
	AddLexeme(new Lexeme("bzp", LX_TOKEN_BZP));
	AddLexeme(new Lexeme("cdc", LX_TOKEN_CDC));
	AddLexeme(new Lexeme("cdp", LX_TOKEN_CDP));
	AddLexeme(new Lexeme("res", LX_TOKEN_RES));

	SortLexemes();
}


bool
LexerObj::NextToken(uint32& Out)
{
	InitNextToken();

	// This loop is exited when a lexeme is found
	for (;;)
	{
		if (EndOfFile)
		{
			Out = Token = LX_CHAR_EOF;
			return true;
		}

		LexmStart = LexmEnd;
		uint8			CharType = Charset[uint8(*LexmEnd)];

		if ((CharType == LX_CHAR_DIGIT) || (*LexmEnd == '.'))
		{
			if (!LexNumber())
				return false;
			Out = Token;
			return true;
		}
		else if (CharType == LX_CHAR_ALPHA)
		{
			if (!LexString())
				return false;
			Out = Token;
			return true;
		}
		else if (CharType == LX_CHAR_SPACE)
		{
			if (!SkipSpaces())
				return false;
		}
		else if (CharType == LX_CHAR_COMMENT)
		{
			if (!SkipLine())
				return false;
		}
		else if (CharType == LX_CHAR_MINUS)
		{
			FWD;
			if (!LexNumber())
				return false;
			Out = Token;
			return true;
		}
		else if (CharType == LX_CHAR_ILLEGAL)	// Illegal characters generate an error
		{
			Out = Token = LX_CHAR_ILLEGAL;
			return false;
		}
		else		// lexeme?
		{
			Out = Token = CharType;
			FWD;
			return true;
		}
	}
}
