//
//
//	File lexer basic class
//
//											(C) 1997 JoOl


#ifndef _LEXER_H_
#define _LEXER_H_


#include "Symbol.h"


#define LEXER_BUFLEN		1024
#define LEXER_LEXMLEN		128


// used for Lexer::Charset
enum
{
	LX_CHAR_SPACE = 0x0,
	LX_CHAR_ALPHA,
	LX_CHAR_DIGIT,
	LX_CHAR_MINUS,
	LX_CHAR_ILLEGAL,
	LX_CHAR_EOF,


	// Leave this at the end of the enumeration
	LX_CHAR_LAST = LX_CHAR_EOF
};



class					Lexer
{

	enum
	{
		LX_SAMEBUF,
		LX_OVERLAPBUF12,
		LX_OVERLAPBUF21,
		LX_LEXEMETOOLONG	// Unused
	};

protected:

	uint8					Charset[256];

	status_t				Status;

	BEntry					Entry;
	BFile					InFile;
	off_t					BytesLeft;

	// buffers related fields
	int32					MainBufLen;
	char*					Buff;
	char*					Buf1;
	char*					Buf2;
	char*					EndBuf1;
	char*					EndBuf2;

	// Lexeme related fields
	int32					LexmBufLen;
	char*					LexemeBuf;
	char*					LexmStart;
	char*					LexmEnd;

	bool					EndOfFile;


#define FWD					if (!Fwd()) return false;
#define BWD					if (!Bwd()) return false;


	BList					Symbols;
	BList*					SearchList;
	void*					SearchItem;
	int						(*SearchFunc)(const void* , const void* );

	static int				CmpSymbols(const void* s1, const void* s2);
	int32					SearchRecur(int32, int32);

	int32					RankOfSymbol(char* );

#pragma export on

	void					SetDefaultCharset();
	void					SetChar(char, uint8);
	void					SetChar(char, char, uint8);


public:

	static bool				PrintSymbol(void* S);
	Symbol*					SearchSymbol(char* );
	bool					AddSymbol(Symbol* );
	void					SortSymbols();
	void					PrintSymbols();


	Lexer(int32, int32);
	~Lexer();

	status_t				GetStatus()
	{ return Status;		};

	bool					Eof()
	{ return EndOfFile;		};

	virtual void			Terminate();

	uint32					LoadBuf1();
	uint32					LoadBuf2();

	void					GetInputSize(off_t& );

	int32					GetConfig();
	int32					GetLexemeLen(int32);
	int32					GetLexemeLen()
	{ return GetLexemeLen(GetConfig());	};
	void					ReadLexeme(char*);		// No allocation
	char*					ReadLexeme();			// With allocation
	char*					GetLexeme();
	void					DeleteLexeme(char* );
	char*					DetachLexeme(char* );

	virtual bool			Fwd();
	virtual bool			Bwd();

#pragma export off

};



#endif	/* _LEXER_H_ */

